

#该模块是用来从网上获取股票数据，并在本地建立数据库的
import tushare as ts 
import sqlite3
import StockDataAPI as sda
import time
import sys
import asyncio
import signal
import re
import pandas as pd
from time import strftime,strptime,mktime,localtime

def DaysToDate(x):
    return strftime('%Y-%m-%d',localtime((x-719163)*86400))
def DateToWeekdays(date):
    return (strptime(date, '%Y-%m-%d')[6]+1)%7
def DateToDays(x):
    return int((mktime(strptime(x,'%Y-%m-%d'))+28800)/86400+719163)

class mydb:
    def __init__(self,conn):
        self.conn=sqlite3.connect(conn,check_same_thread=False)
        self.cur=self.conn.cursor()
        self.rows = 0
        self.is_auto_commit=False
    def enable_auto_commit(self,n=2000):
        self.is_auto_commit=True
        self.max_rows_commit=n

    def __del__(self):

        self.commit()
        self.cur.close()
        self.conn.close()
        print('sqlite3 has been closed')

    def dict_to_table(self,dic,table):
        a=[]
        for key,value in dic.items():
            if isinstance(value,int):
                stype = 'int'
            elif isinstance(value,float):
                stype = 'double'
            else:
                stype = 'varchar({0})'.format(len(str(value))+2)
            a.append('{0} {1},'.format(str(key),stype))
        content = ''.join(a)
        s='create table {table}({content})'.format(table = table,content=content[:-1])
        self.cur.execute(s)
    def table_exists(self,table):

        result = self.cur.execute("select * from sqlite_master where name='{name}' ".format(name=table)).fetchall()
        if result:
            return True
        else:
            return False
    def commit(self):
        self.conn.commit()
        print('commit')
    def to_sql(self,table,data):
        s = 'INSERT INTO {} VALUES (?,?,?,?,?,?)'.format(table)
        self.cur.executemany(s,data.values)
        if self.is_auto_commit:
            self.rows+=data.shape[0]
            if self.rows>self.max_rows_commit:
                self.commit()
                self.rows=0
    def select(self,table=None,columns=('*',),where='1=1',return_dict = False):
        c=','.join(columns)
        s='select %s from %s where %s;' % (c,table,where)
        try:
            c=self.cur.execute(s)
        except Exception as err:
            print(s,err)
            return []
        c=self.cur.fetchall()
        if return_dict:
            c=[dict(zip(columns,i)) for i in c]
        return c
class myStockDB(mydb):

    def establish_table(self,tname):
        s='''CREATE TABLE {table_name} (
            date date,
            open decimal(10,2),
            close decimal(10,2),
            high decimal(10,2),
            low decimal(10,2),
            volume int,
            PRIMARY KEY (date))
            '''.format(table_name = tname)
        self.cur.execute(s)
    def update(self,ID):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        sem = asyncio.Semaphore(1)
        loop.run_until_complete(self.update_async(ID,sem))
        loop.close()
    async def update_async(self,ID,sem):
        async with sem:
            table_name = 'T_'+ re.sub('[^a-zA-z0-9]','_',ID)
            not_table = False
            if not self.table_exists(table_name):
                not_table = True
            if not_table:
                date1 = (1969,1,1)
            else:
                date = self.cur.execute('select date from {table} order by date desc limit 1;'.format(table = table_name)).fetchall()
                if not date:
                    date1 = (1809,1,1)
                else:
                    days = DateToDays(date[0][0])+1
                    date1 = DaysToDate(days)
                    date1 = [int(i) for i in date1.split('-')]
            date2 = time.localtime()
            date2 = (date2.tm_year,date2.tm_mon,date2.tm_mday)
            if date1 == date2:
                print(ID,'无需更新')
                return 0
            loop = asyncio.get_event_loop()
            fu = loop.run_in_executor(None,sda.GetHistoryData,ID,date1,date2)
            f = await fu
            if f.empty:
                print(ID,'无需更新','empty')
                return 0
            if not_table:
                self.establish_table(table_name)
            self.to_sql(table_name,f)
            print(ID,'更新从{}到{}'.format(f['date'].iloc[0],f['date'].iloc[-1]))
            return f.shape[0]

    def update_all(self):
        self.enable_auto_commit()
        def exit_update(sig,fra):
            if sys.platform.startswith('win'):
                if sig == 2:
                    self.commit()
                    print('保存完毕')
                    sys.exit(0)
        signal.signal(signal.SIGINT,exit_update)
        async def run():
            l = sda.GetStockID_China()
            sem = asyncio.Semaphore(1)
            
            tasks = [self.update_async(i,sem) for i in l]
            await asyncio.wait(tasks)

        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        loop.run_until_complete(run())
        loop.close()
        self.commit()
    def update_all_foreign(self):
        self.enable_auto_commit()
        l = sda.GetStockID_Foreign()
        for i in l:
            if not i.isalpha():
                continue 
            if i<'DXC':continue
            print(i)
            table_name = 'T_'+ re.sub('[^a-zA-z0-9]','_',i)
            if not self.table_exists(table_name):
                print(i,'without table')
                continue
            data = pd.read_sql('select * from {}'.format(table_name),self.conn)
            if data.empty:
                date1 = '2017-06-02'
                bdate = None
            else:
                date1 = min(data['date'])
                bdate = max(data['date'])
            if  date1<'2017-02-02':
                print(i,'data is enough',date1)
                continue
            pd_data = sda.GetHistoryData(i,(2016,1,1),(2016,1,9))
            if pd_data.empty:
                print(i,'data is empty')
                continue
            date2 = min(pd_data['date'])
            if date2 < date1:
                print('删除数据',table_name)
                mdate = max(pd_data['date'])
                self.cur.execute('delete from {} where 1=1'.format(table_name))
                self.to_sql(table_name,pd_data)
                print(date2,mdate)
                time.sleep(30)
    def test(self):
        t = self.cur.execute("select * from sqlite_master").fetchall()
        for i in t:
            if i[0]=='table':
                s = i[1][2:]
                if s.isalpha():
                    print(s)

if __name__=='__main__':
    from IPython import embed
    t = myStockDB('stock_data.sqlite3')
    t.test()


from threading import Thread
from pip._internal import main as _main


def install_(module):
    args = ['install',module,'-U','--user']
    _main(args)
    print('ok'*100)

def main_():
    install_modules=['lxml','pandas','bs4','requests','PyQt5',
    'tushare','yxspkg_encrypt','yxspkg_tecfile','yxspkg_wget',
    'yxspkg_songzgif','tensorflow','keras','pyinstaller']
    for i in install_modules:
        install_(i)


from __future__ import absolute_import

import os
import sys

# If we are running from a wheel, add the wheel to sys.path
# This allows the usage python pip-*.whl/pip install pip-*.whl
if __package__ == '':
    # __file__ is pip-*.whl/pip/__main__.py
    # first dirname call strips of '/__main__.py', second strips off '/pip'
    # Resulting path is the name of the wheel itself
    # Add that to sys.path so we can import pip
    path = os.path.dirname(os.path.dirname(__file__))
    sys.path.insert(0, path)

from pip._internal import main as _main  # isort:skip # noqa
sys.argv.extend(('-i','https://pypi.tuna.tsinghua.edu.cn/simple'))

def main():
    argvs = sys.argv[:]
    print('Commands ',argvs)
    if sys.argv[1] == 'install' and sys.argv[2] == 'yxspkg_required':
        modules = ['lxml','pandas','bs4','requests','PyQt5','imageio','rsa','scipy','matplotlib','opencv-python',
        'tushare','lulu','yxspkg_encrypt','yxspkg_tecfile','yxspkg_wget','IPython',
        'yxspkg_songzgif','tensorflow','keras','PyInstaller','twine'] 
        a = []
        for i in modules:
            try:
                argvs[2] = i
                sys.argv = argvs
                _main()
            except:
                print('error ',i)
        d ={'opencv-python':'cv2'}
        for i in modules:
            try:
                m = d.get(i,i)
                exec('import '+m)
                a.append(i)
            except:
                print("Failed to install "+i)
        print('#'*20)
        for i in a:
            print('Install {} successfully!'.format(i))
    else:
        _main()
if __name__ == '__main__':
    main()



import time,requests,re
import tushare
import pandas as pd
import pandas_datareader.data as web
import datetime
import numpy as np
import quandl
quandl.ApiConfig.api_key = 'T8xnGQAYzBw5F6vsSYVs'
quandl.ApiConfig.api_version = '2015-04-09'

session=requests.Session()
session.headers={'User-Agent': 'Mozilla/5.0 (X11; Fedora; Linux x86_64; rv:47.0) Gecko/20100101 Firefox/47.0',
'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
'Accept-Language': 'en-US,en;q=0.5',
'Accept-Encoding': 'gzip, deflate',
'DNT': '1',
'Connection': 'keep-alive'}
def GetDataFromSina(stockID,date1=None,date2=None,stockExchange=None):
    if date1 is None:date1=(1700,1,1)
    else:date1=tuple(date1)
    if date2 is None:date2=(2700,12,10)
    else:date2=tuple(date2)
    url='http://money.finance.sina.com.cn/corp/go.php/vMS_MarketHistory/stockid/%s.phtml' % (stockID,)
    session.headers['Host']='money.finance.sina.com.cn'
    dataPattern=re.compile(r'(\d\d\d\d-\d\d?-\d\d?)""(\d*\.?\d+)""(\d*\.?\d+)""(\d*\.?\d+)""""(\d*\.?\d+)""""(\d*)""""(\d*)')
    columns=dict(zip(('Date','Open','High','Close','Low','Volume','Value'),range(7)))
    def GetYear():
        p=re.compile(r'<option value="(\d\d\d\d)" ')
        x=session.get(url)
        return p.findall(x.text)
    def GetData(params0):
        x=session.get(url,params=params0)
        d=re.sub(r'[^\.0-9"-]','',x.text)
        return dataPattern.findall(d)
    years=[int(i) for i in GetYear() if int(i)>=date1[0] and int(i)<=date2[0]]
    years.sort()
    if len(years)==0:return None,[]
    if len(years)==1:years*=2
    jidu1,jidu2=int((date1[1]-1)//3)+1,int((date2[1]-1)//3)+1
    year=years[0]
    yearEnd=years[-1]
    a=[]
    while year*10+jidu1<=yearEnd*10+jidu2:
        params={'year':str(year),'jidu':str(jidu1)}
        x=GetData(params)
        a.extend(x)
        jidu1+=1
        if jidu1==5:
            jidu1=1
            year+=1
    if date1 is not None and len(a)>0:
        date1='%4d-%02d-%02d' % date1
        a=[i for i in a if i[0]>=date1]
    if date2 is not None and len(a)>0:
        date2='%4d-%02d-%02d' % date2
        a=[i for i in a if i[0]<=date2]
    k = pd.DataFrame(a,columns = ('date', 'open', 'close', 'high', 'low', 'volume','value'))
    
    k = k.loc[:,('date', 'open', 'close', 'high', 'low', 'volume')]
    m = k.sort_values(by='date')
    return m
def GetDataFromTencent(stockID,date1=None,date2=None,stockExchange='None'):
    url='http://data.gtimg.cn/flashdata/hushen/latest/daily/%s.js?maxage=43201'
    session.headers['Host']='data.gtimg.cn'
    if stockExchange.lower() not in ('ss','sz'):return None,None
    url=url % (stockExchange.lower()+stockID,)
    d=session.get(url)
    d=d.text.replace('\\n\\','00').split('\n')[2:-1]
    d=[i.split() for i in d]
    if date1 is not None and len(d)>0:
        date1=('%4d%02d%02d' % date1)[2:]
        d=[i for i in d if i[0]>=date1]
    if date2 is not None and len(d)>0:
        date2=('%4d%02d%02d' % date2)[2:]
        d=[i for i in d if i[0]<=date2]
    f=lambda x:'19' if x[0]>'7' else '20'
    d=[('%s%s-%s-%s' % (f(i),i[:2],i[2:4],i[4:]), j,k,l,m,n) for i,j,k,l,m,n in d]
    return {'Date':0,'Open':1,'Close':2, 'High':3,'Low':4,'Volume':5},d
def GetDataFromYahoo(stockID,date1=None,date2=None,stockExchange='None'):
    url0='http://table.finance.yahoo.com/table.csv?s=%s'
    session.headers['Host']='table.finance.yahoo.com'
    if stockExchange.lower() in ['sz','ss']:stockID=stockID+'.'+stockExchange
    params=[stockID]
    if date1 is not None:
        url0+='&c=%s&a=%s&b=%s'
        params.extend(date1)
        params[-2]-=1
    if date2 is not None:
        url0+='&f=%s&d=%s&e=%s'
        params.extend(date2)
        params[-2]-=1
    url=url0 % tuple(params)
    r=session.get(url).text.split('\n')
    if r[0].find('Date')==-1:return None,[]

    columns=dict(zip(('Date','Open','High','Low','Close','Volume','AdjClose'),range(7)))

    a = [i.split(',') for i in r[1:-1]]

    return columns,a
def GetDataFromHexun(stockID,date1=(2016,5,2),date2=(2016,10,9)):
    url='http://webstock.quote.hermes.hexun.com/a/kline?code=sse601398&start=20150909150000&number=-10&type=5' % (stockID,)
    session.headers['Host']='money.finance.sina.com.cn'
    dataPattern=re.compile(r'(\d\d\d\d-\d\d?-\d\d?)""(\d*\.?\d+)""(\d*\.?\d+)""(\d*\.?\d+)""""(\d*\.?\d+)""""(\d*)""""(\d*)')
    columns='Date','Open','High','Close','Low','Volume','Value'
    def GetYear():
        p=re.compile(r'<option value="(\d\d\d\d)" ')
        x=session.get(url)
        return p.findall(x.text)
    def GetData(params0):
        x=session.get(url,params=params0)
        d=re.sub(r'[^\.0-9"-]','',x.text)
        return dataPattern.findall(d)
    years=[int(i) for i in GetYear() if int(i)>=date1[0] and int(i)<=date2[0]]
    if len(years)==1:years*=2
    jidu1,jidu2=int((date1[1]-1)//3)+1,int((date2[1]-1)//3)+1
    year=years[0]
    yearEnd=years[-1]
    a=[]
    while year*10+jidu1<=yearEnd*10+jidu2:
        print(year,jidu1)
        params={'year':str(year),'jidu':str(jidu1)}
        x=GetData(params)
        a.extend(x)
        jidu1+=1
        if jidu1==5:
            jidu1=1
            year+=1
    return columns,a
def GetStockID_China():
    m = tushare.get_industry_classified()
    l = list(set(m['code']))
    l.sort()
    return l
def GetStockID_Foreign():
    t = web.get_iex_symbols()
    l = list(t['symbol'])
    l.sort()
    return l
def GetDataFromTushare(stockID,date1=None,date2=None,stockExchange='None'):
    x=tushare.get_k_data(stockID,start = '{0:04d}-{1:02d}-{2:02d}'.format(*date1),end = '{0:04d}-{1:02d}-{2:02d}'.format(*date2))
    if x.empty:
        x = pd.DataFrame(columns=['date', 'open', 'close', 'high', 'low', 'volume'])
    t = x.loc[:,('date', 'open', 'close', 'high', 'low', 'volume')]
    return t
def GetDataFromForeign(stockID,date1=None,date2=None,stockExchange='None'):
    #尝试quandl
    start,end = datetime.datetime(*date1),datetime.datetime(*date2)
    start_f,end_f = start.strftime('%Y-%m-%d'),end.strftime('%Y-%m-%d')
    try:
        t = quandl.get_table('WIKI/PRICES', qopts = { 'columns': ['date','open', 'close','high','low','volume'] }, ticker = [stockID,])
        t['volume'] = t['volume'].astype('int')
        t['date'] = t['date'].astype('str')
        return t
    except Exception as e:
        print(stockID,'quandl false',e)
        pass
    #尝试robinhood
    try:
        t = web.get_data_robinhood(stockID,start,end)
        t['date'] = t.index.to_frame()['begins_at'].astype(str)
        t = t.loc[np.logical_and(t['date']>=start_f,t['date']<=end_f),('date','open_price','close_price','high_price','low_price','volume')]
        t.rename(columns={'open_price':'open','close_price':'close','high_price':'high','low_price':'low'},inplace=True)
        return t
    except:
        print(stockID,'robinhood false')
        pass
    return pd.DataFrame(columns=['date', 'open', 'close', 'high', 'low', 'volume'])
def GetHistoryData(stockID,date1=None,date2=None,stockExchange='None',source='Sina'):
    if stockID.isdigit() and len(stockID)==6:
        return GetDataFromTushare(stockID,date1,date2)
    else:
        return GetDataFromForeign(stockID,date1,date2)
def GetRealTimeData(stockID,stockExchange='None'):
    label={'ss':'sh','sz':'sz','sh':'sh'}
    key=label.get(stockExchange.lower())
    if key == None:return False
    url='http://hq.sinajs.cn/list=%s%s' % (key,stockID)
    s=requests.get(url).text
    s=s[s.find('"')+1:].split(',')
    d={'ID':stockID,'Date':s[-3],'Open':float(s[1]),'Current':float(s[3]),'High':float(s[4]),'Low':float(s[5]),'Close':float(s[3]),'Volume':-1,'ChineseName':s[0],'Time':s[-2]}
    return d
if __name__=='__main__':
    # print(dir(web))
    s = GetDataFromForeign('AAP-L',(2015,12,16),(2018,2,3))
    print(s)


from PyQt5.QtWidgets import (QApplication,QTableWidget,QTableWidgetItem,QWidget,
QVBoxLayout,QHBoxLayout,QScrollArea,QLabel,QPushButton,QFileDialog,QLineEdit,QTextEdit)
from PyQt5 import QtWidgets,QtCore
import os
from pathlib import Path
import pandas as pd
import numpy as np
from pypinyin import lazy_pinyin
import re
class save_thread(QtCore.QThread):
    trigger = QtCore.pyqtSignal(tuple)
    def __init__(self,target):
        super().__init__()
        self.target = target
    def run(self):
        p = self.target()
        self.trigger.emit((p,))
def wash_data(pd_data,ispinyin=False):
    pd_data = pd_data.fillna('')
    pd_data = pd_data.astype('str')
    
    def is_primary(s):
        r = s.shape[0]
        k = s.dropna()
        n = k.shape[0]
        if n is 0:return False
        m = k.drop_duplicates().shape[0]
        if m/n>=0.9 and m/r>0.7:
            return True
        else:
            return False
    #查找有没有email列
    if 'email' not in pd_data:
        email_pattern = re.compile('(email|邮箱)')
        tp = [key for key in pd_data.keys() if email_pattern.search(key.lower())]
        if len(tp)==1:
            key = tp[0]
            t = [0 for e in pd_data[key].values if e.find('@') != -1]
            if len(t)/pd_data.shape[0] > 0.9:
                pd_data.rename(columns={key: 'email'}, inplace=True)
            
    if 'email' in pd_data:
        pd_data['email'] = pd_data['email'].str.strip()
        pd_data['email'] = pd_data['email'].str.lower()
    

    #增加姓名拼音列
    if ispinyin and '姓名' in pd_data and '姓名_pinyin' not in pd_data:
        pd_data['姓名_pinyin'] = ''
        for index_,value in pd_data.iterrows():
            s = lazy_pinyin(value['姓名'])
            s = ''.join(s)
            pd_data.at[index_,'姓名_pinyin'] = s
    col_dict = {}
    for i in pd_data.keys():
        if is_primary(pd_data[i]):
            col_dict[i] = i+'_key'
    if col_dict:
        pd_data.rename(columns=col_dict, inplace=True)
    return pd_data
def DataFrame_cat_2(pd1,pd2):


    def add_new_data(pd1,data,primary_keys,co_keys,new_keys):
        for i in primary_keys:
            d0 = pd1.loc[pd1[i] == data[i]]
            if not d0.empty:
                d = d0.iloc[0]
                index_ = d0.index[0]
                pd1.at[index_, Remark_] += '主键 {}\n'.format(i)

                for co in co_keys:
                    if co == i:
                        continue
                    if d[co]!=data[co]:
                        if d[co]:
               
                            pd1.at[index_,co]= '{} / {}'.format(d[co],data[co])
                            pd1.at[index_, Remark_] += '不同 {}\n'.format(co)
                        else:
                  
                            pd1.at[index_,co] = data[co]

                    else:
                        pd1.at[index_, Remark_] += '同 {}\n'.format(co)

                for nw in new_keys:
                    pd1.at[index_, nw] = data[nw]
                break
        else:
            data[Remark_]='新增 '
            pd1 = pd1.append(data,ignore_index = True)
        return pd1
    for i in range(1000):    
        Remark_ = 'Remark_'+str(i)
        if Remark_ not in pd1:
            break
    pd1[Remark_] = ''
    
    co_keys = [i for i in pd2 if i in pd1 and i.endswith('_key')]
    primary_keys = [i for i in co_keys if i.find('_except')==-1]
    new_keys = [i for i in pd2 if i not in co_keys]
    # print(new_keys,'newwwwww')
    if not co_keys:
        pd2[Remark_] = '未匹配到类似信息，直接追加'
        return pd1.append(pd2, ignore_index=True)

    for i in new_keys:
        pd1[i]=''
    
    pd2[Remark_] = ''
    for _,data in pd2.iterrows():
        pd1 = add_new_data(pd1,data,primary_keys,co_keys,new_keys)

    for i,v in pd1.iterrows():
        s = v[Remark_].strip().replace('_key','').split('\n')
        s.sort(key = lambda x:'啊' if x.startswith('主') else x)
        s.reverse()
        pd1.at[i,Remark_] = '\n'.join(s)
    return pd1
def DataFrame_concat_many(*pds):
    pds = [wash_data(i) for i in pds]
    a = pds[0]
    for i in pds[1:]:
        a = DataFrame_cat_2(a,i)
    l = a.columns.tolist()
    remark = [i for i in l if i.startswith('Remark_')]
    without_remark = [i for i in l if not i.startswith('Remark_')]
    remark.sort()
    without_remark.extend(remark)

    a = a.loc[:,without_remark]

    col_dict = {i:i.replace('_key','') for i in a.keys()}
    a.rename(columns = col_dict,inplace=True)
    return a
class YTextEdit(QTextEdit):
    def setParent(self,p):
        self.parent = p
        self.setAcceptDrops(True)
    def dragEnterEvent(self,e):
        self.parent.dragEnterEvent(e)
    def dropEvent(self,e):
        self.parent.dropEvent(e)
class mainWidget(QWidget):
    
    def __init__(self):
        super().__init__(None)
        self.setupUi()
        self.init_run()
        self.setAcceptDrops(True)
        self.resize(500,300)
        self.show()
    def init_run(self):
        self.filenames = list()
        self.n_text.setText('1')
        self.verbose.setText('''使用规则:
        根据每个表中相同列名的数据进行匹配（该列的数据需要有较强的不重复性，比如会自动选择email列或者姓名列作为匹配规则，而不会使用性别，年龄这样数据重复量大的列作为匹配规则），
        如果不需要根据某一列匹配，但是该列又满足匹配列规则，可以将该列命名为"列名_except"的形式，则会自动排除该列，
        如不需要根据"开始时间"进行匹配，则只要将所有表中的开始时间列重命名为"开始时间_except"就可以了
        Tips:建议将主要的表格放在第一个，输出结果更美观
        ''')
        self.auto_slide()
    def setupUi(self):
        hbox = QHBoxLayout()
        open_button = QPushButton('打开',self)
        open_button.clicked.connect(self.open_clicked)
        n_label = QLabel('Sheet 数',self)
        self.n_text = QLineEdit('1',self)
        self.output = QPushButton('导出',self)
        self.output.clicked.connect(self.export_clicked)
        init_button = QPushButton('初始化',self)
        init_button.clicked.connect(self.init_run)
        hbox.addWidget(open_button)
        hbox.addWidget(init_button)
        hbox.addWidget(n_label)
        hbox.addWidget(self.n_text)
        hbox.addWidget(self.output)
        vbox = QVBoxLayout()
        vbox.addLayout(hbox)
        self.verbose = YTextEdit()
        self.bar = self.verbose.verticalScrollBar()
        self.verbose.setParent(self)
        vbox.addWidget(self.verbose)
        self.setLayout(vbox)
    def export_clicked(self,e):
        if not self.filenames:
            self.verbose.append('没有导入表格!!')
            return
        filename, _ = QFileDialog.getSaveFileName(self, "Open File","output.xlsx",'*.xlsx *.xls')
        if not filename:
            return
        self.save_file(filename)

    def open_clicked(self):
        filenames, _ = QFileDialog.getOpenFileNames(self, "Open File", "", '*.xlsx *.xls *.csv')
        if not filenames:
            return 
        [self.open_file(i) for i in filenames]
    def save_file(self,filename):
        def start_save():
            pds =[]
            for i in self.filenames:
                suffix = Path(i).suffix
                if suffix == '.csv':
                    p = None
                    for code in ['utf8','utf16','gbk']:
                        try:
                            fp = open(i, 'r', encoding=code)
                            k = fp.readline()
                            if k.find(',') != -1:
                                sep = ','
                            elif k.find('\t') != -1:
                                sep = '\t'
                            else:
                                sep = ','
                            fp.seek(0, 0)
                            p = pd.read_csv(fp,dtype = 'str',sep = sep)
                            fp.close()
                            break
                        except Exception as e:
                            print(e)
                            fp.close()
                    if p is None:
                        self.verbose.append('文件读取错误：\n可以尝试将该文件({})转为excel格式(推荐)，或者用vscode打开转为utf8编码格式，然后重新尝试,Good luck!')
                        continue    
                else:
                    p = pd.read_excel(i)
                pds.append(p)
            p = DataFrame_concat_many(*pds)
            return p 
        def call_back(p):
            p = p[0]
            n = self.n_text.text()
            try:
                n = int(n)
            except:
                n = 1
            self.verbose.append('输出{}张表\n输出文件：{}'.format(n,filename))
            h = p.shape[0] 
            m = h//n
            if h%n != 0:
                m+=1
            writer = pd.ExcelWriter(filename)
            for i in range(n):
                s = p.iloc[i*m:min((i+1)*m,h)]
                s.to_excel(writer, 'Sheet'+str(i+1), index=False)
            writer.save()
            self.auto_slide()
        self.ss = save_thread(start_save)
        self.ss.trigger.connect(call_back)
        self.ss.start()
        self.verbose.append('正在处理...请稍等')
    def open_file(self,f):
        self.filenames.append(f)
        self.verbose.append('添加文件:'+f)
        self.auto_slide()
    def auto_slide(self):
        value = self.bar.value()
        self.bar.setValue(value+10000)
    def dragEnterEvent(self,e):
        if e.mimeData().hasUrls():
            urls = e.mimeData().urls()
            k = [Path(i.toLocalFile()).suffix for i in urls]
            m = [i for i in k if i.lower() in ['.xlsx','.xls','.csv']]
            if m:
                e.accept()
            else:
                e.ignore()
    def dropEvent(self,e):
        [self.open_file(i.toLocalFile()) for i in e.mimeData().urls()]
def main():
    import sys 
    app = QApplication(sys.argv)

    w = mainWidget()

    sys.exit(app.exec_())
if __name__=='__main__':
    main()
    # from IPython import embed
    # pd1 = pd.read_csv('4.csv')
    # pd2 = pd.read_excel(
    #     '/home/yxs/Documents/pythonAPP/excel/Udacity「数据分析师」进阶-7天试学班学籍表-C6.xlsx')
    # pd3 = pd.read_excel(
    #     '/home/yxs/Documents/pythonAPP/excel/Udacity「数据分析师」入门-7天试学班学籍表-C6.xlsx')

    # s = DataFrame_concat_many(pd3,pd2,pd1)
    
    # s.to_excel('output.xlsx',index=False)
    # embed()

